/// @addtogroup dynsys
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file BasicC2Taylor.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file constitutes a part of the CAPD library, 
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details. 

/* Author: Daniel Wilczak 2003-2005 */

#ifndef _CAPD_DYNSYS_BASICC2TAYLOR_H_ 
#define _CAPD_DYNSYS_BASICC2TAYLOR_H_ 

#include <string>
#include "dynsys/BasicTaylor.h"

namespace capd{
namespace dynsys{

template<typename MapT>
class BasicC2Taylor : public virtual BasicTaylor<MapT>{
public:
  typedef MapT MapType;
  typedef typename MapT::FunctionType FunctionType;
  typedef typename MapType::MatrixType MatrixType;
  typedef typename MatrixType::RowVectorType VectorType;
  typedef typename MatrixType::ScalarType ScalarType;
  typedef capd::vectalg::Norm<VectorType,MatrixType> NormType;
  typedef typename MapType::C2CoeffType C2CoeffType;

  BasicC2Taylor(MapType& vectorField, int order, const ScalarType& step);
  ~BasicC2Taylor();
   
  VectorType operator()(const VectorType&, MatrixType&, C2CoeffType&);

  // If someone knows how to use 'using BasicTaylor<MapType>::operator()' in the borland compiler?:(
  VectorType operator()(const VectorType& v) const{
    return BasicTaylor<MapType>::operator()(v);
  }
  VectorType operator()(const VectorType& v, MatrixType& der) const{
    return BasicTaylor<MapType>::operator()(v,der);
  }
  void setOrder(int newOrder);

  using BasicTaylor<MapType>::setStep;

  using BasicTaylor<MapType>::getField;
  using BasicTaylor<MapType>::getOrder;
  using BasicTaylor<MapType>::getStep;
  using BasicTaylor<MapType>::getCoeff;
  using BasicTaylor<MapType>::getMatrixCoeff;

protected:
  BasicC2Taylor(const BasicC2Taylor &t) : BasicTaylor<MapT>(t){}
  void operator=(const BasicC2Taylor&){}

  using BasicTaylor<MapType>::computeMatrixCoeff;
  using BasicTaylor<MapType>::m_vField;
  using BasicTaylor<MapType>::m_order;
  using BasicTaylor<MapType>::m_step;
  using BasicTaylor<MapType>::m_coeff;
  using BasicTaylor<MapType>::m_matrixCoeff;
  using BasicTaylor<MapType>::m_dimension;
  using BasicTaylor<MapType>::m_F;

  C2CoeffType* m_c2MatrixCoeff;
  void computeC2Coeff(const VectorType& x, const MatrixType& V, const C2CoeffType& H, int order) const;
}; // the end of class BasicC2Taylor


// ------------------ inline definitions -------------

template <typename MapType>
inline BasicC2Taylor<MapType>::~BasicC2Taylor()
{
  delete [] m_c2MatrixCoeff;
}

}} // the end of the namespace capd::dynsys

#endif // _CAPD_DYNSYS_BASICC2TAYLOR_H_ 

/// @}
