//////////////////////////////////////////////////////////////////////////////
//   Package:          CAPD
//   Subpackage:
/// @addtogroup intervals
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file doubleInterval.cpp
///
/// @author Tomasz Kapela   @date 11-01-2006
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) Tomasz Kapela 2006
//
// This file constitutes a part of the CAPD library, 
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details. 

#include <cmath>

#include "rounding/doubleRounding.h"

#include "interval/doubleInterval.h"

#include "interval/interval.h"

using std::log;

typedef capd::rounding::DoubleRounding DoubleRounding;

template class capd::intervals::Interval<double, DoubleRounding>;

interval pi = interval::pi();

template interval capd::intervals::diam<double, DoubleRounding>(const interval& ); 
template bool  capd::intervals::intersection<double, DoubleRounding>( interval A_iv1, 
                    interval A_iv2,
                    interval &A_rIntersection); 
template interval capd::intervals::intervalHull<double, DoubleRounding>(const interval & A_iv1, 
                                                                        const interval & A_iv2);
template void capd::intervals::split<double, DoubleRounding>( interval & A_iv, interval & A_rMid, 
                                                              double & A_diam);
template interval capd::intervals::power<double, DoubleRounding> (const interval & x, int n);
template interval capd::intervals::power <double, DoubleRounding>(const interval & a, 
                                                                  const interval & b);
template interval capd::intervals::sqrt <double, DoubleRounding>(const interval &x);
template interval capd::intervals::sin <double, DoubleRounding>(const interval& x);
template interval capd::intervals::cos <double, DoubleRounding>(const interval& x);
template interval capd::intervals::tan <double, DoubleRounding>(const interval& x);
template interval capd::intervals::cot <double, DoubleRounding>(const interval& x);
template interval capd::intervals::atan <double, DoubleRounding>(const interval& x);
template interval capd::intervals::asin <double, DoubleRounding>(const interval& x);
template interval capd::intervals::acos <double, DoubleRounding>(const interval& x);
template interval capd::intervals::sinh <double, DoubleRounding>(const interval& x);
template interval capd::intervals::cosh <double, DoubleRounding>(const interval& x);
template interval capd::intervals::tanh <double, DoubleRounding>(const interval& x);
template interval capd::intervals::coth <double, DoubleRounding>(const interval& x);
template interval capd::intervals::exp <double, DoubleRounding>(const interval & x);
template interval capd::intervals::log <double, DoubleRounding>(const interval& x);
template interval capd::intervals::solveAffineInclusion<double, DoubleRounding>(const interval & a, 
                                               const interval & p, 
                                               const interval & c);
template interval capd::intervals::solveAffineInclusion<double, DoubleRounding>(const interval & a, 
                                               const interval & p, 
                                               const interval & c,
                                               int & dir);
 
 
//////////////////////////////////////////////////////////////////////////
//         Arithmetic operators
//////////////////////////////////////////////////////////////////////////
#ifndef __INTERVAL_SPEED_OPTIMIZED__

template interval capd::intervals::operator +<double, DoubleRounding>(const interval & A_iv1, const interval & A_iv2);
template  interval capd::intervals::operator -<double, DoubleRounding>(const interval & A_iv1, const interval & A_iv2);
template interval  capd::intervals::operator *<double, DoubleRounding>(const interval & A_iv1, 
                                      const interval & A_iv2);
template interval capd::intervals::operator /<double, DoubleRounding>(const interval & A_iv1, 
                                     const interval & A_iv2);
template std::ostream & capd::intervals::operator << <double, DoubleRounding>(std::ostream & s, 
                                   const interval & A_iv);
template std::istream &capd::intervals::operator>> <double, DoubleRounding>(std::istream &inp, interval &i);
template interval capd::intervals::add<double, DoubleRounding>(const interval & A_iv, const double & A_x);
template interval capd::intervals::substract<double, DoubleRounding>(const interval & A_iv, const double &A_x);
template interval capd::intervals::substract<double, DoubleRounding>(const double  &A_x, const interval & A_iv);
template interval capd::intervals::multiply<double, DoubleRounding>(const interval & A_iv, const double & A_x);
template interval capd::intervals::divide<double, DoubleRounding>(const interval & A_iv, 
                                            const double & A_x);
template  interval capd::intervals::divide<double, DoubleRounding>(const double & A_x, const interval & A_iv);
#endif  //__INTERVAL_SPEED_OPTIMIZED__
											   
/// @}

